@@ -1,7 +1,7 @@
 
 package Log::Log4perl::AutoCategorize;
 use strict;
-our $VERSION = "0.03_51";
+our $VERSION = 0.02;
 
 use Carp;
 use IO::File;
@@ -381,15 +381,20 @@ END {
 }
 
 ###################
-# optimizer stuff, only for 5.8
+# optimizer stuff
 # __END__ # uncomment __END__ to use in 5.6.x, and accept slowdown
 
-# silence 'Subroutine redefined' warnings due to B::Generate
-local $SIG{__WARN__};	# localize outside BEGIN block
+
+# these dont work to silence the warnings
+{ package optimizer;   no warnings 'redefine' }
+{ package B::Generate; no warnings 'redefine' }
+{ package DynaLoader; no warnings 'redefine' }
+no warnings 'redefine';
+
 BEGIN {
-    $SIG{__WARN__} = sub { 
-	return if $_[0] =~ /Subroutine B(::\w+)+ redefined/;
-	warn $_[0];
+    local $SIG{__WARN__} = sub { 
+	print "got: $_, @_";
+	return if /B::\w+ redefined/;
     };
 }
 
@@ -417,9 +422,8 @@ use optimizer 'extend-c' => sub {
     # Log::Log4perl::AutoCategorize or $Alias.  All others end the chain.
 
     my $class = '';
-    eval { $class = $opp->sv->PV };	# sets $class unless $@
-    return unless $class and $Alias and $MyPkg;
-
+    eval { $class = $opp->sv->PV };
+    # if above failed, $class wont be set, so no $@ check is needed
     return unless $class eq $Alias or $class =~/^$MyPkg/;
 
     $DB::single = 1 if $opt->{D};
@@ -510,11 +514,10 @@ Log::Log4perl::AutoCategorize - extended Log::Log4perl logging
 
 =head1 ABSTRACT
 
-Log::Log4perl::AutoCategorize extends Log::Log4perls (l4p) easy mode,
+Log::Log4perl::AutoCategorize extends Log::Log4perl's easy mode, 
 adding 2 main features;
 
   1. extended, automatic, transparent categorization capabilities
-    a. logging category <- package.subroutine.loglevel.linenumber
   2. runtime information useful for:
     a. test-coverage assessment
     b. managing your logging config.
@@ -533,16 +536,13 @@ for comparison;
 
 =head1 SYNOPSIS
 
-  # use pre-defined logging - to stdout
-  use Log::Log4perl::AutoCategorize;
-
-  # OR separate your log-config (standard practice)
-  use Log::Log4perl::AutoCategorize ( initfile => $filename );
-
-  # OR define your config inline (for development)
   use Log::Log4perl::AutoCategorize
     (
      alias => 'Logger', # shorthand class-name alias
+     # you can initialize in use statement
+     #  1st way gives separation of code from config
+     #  2nd way is good for demonstration and early development
+     initfile => $filename,
      initstr => q{
          # see Log4perl docs to understand these directives
 	 log4perl.rootLogger=DEBUG, A1
@@ -593,8 +593,9 @@ for comparison;
 
 Before diving in, a few notes:
 
-This is not intended to document Log4perl (aka: l4p,) which already
-has excellent documentation.
+This is not intended to document Log4perl (abbrev for Log::Log4perl);
+that documentation is a good tutorial, and is quite thorough and
+complete.
 
 In this document, I use B<Logger> as the I<official> shorthand for
 Log::Log4perl::AutoCategorize, as it gets tedious and verbose to
@@ -611,14 +612,11 @@ brick into a more hat-like shape.
 This module requires perl 5.8, as the optimizer is unsupported in 5.6.
 You can use this module IFF you uncomment the __END__ before the
 optimization stuff is done (at bottom of file), but runtime costs are
-considerable (40% overhead), so Im reluctant to add this as an option.
-That said, l4p install scripts adjust the code to the perl version
-(ex, they revert from 'our $var' syntax for 5.5.3, since its a 5.6
-feature).  I would accept a patch to do similar..
+considerable (40% overhead), so I wont add this as an option.
 
 =head1 AutoCategorization
 
-The primary feature of this module is to extend Log4perls flexibility
+The primary feature of this module is to extend Log4perl's flexibility
 by automatically and transparently creating logging categories for each
 invocation in your application code.
 
@@ -634,19 +632,19 @@ uses that category to properly handle the message at runtime.
 
 
 This module extends the $logCat with info thats obtained at runtime,
-leveraging L4Ps filtering mechanisms by giving more detail upon which
-to filter.
+leveraging Log::Log4perl's categorization of log-events by giving more
+detail upon which to filter.
 
     $logcat = "$package.$subname.$loglevel.$linenum";
 
-L4Ps bubble-up behavior is still in effect; if your log-config never
-specifies anything beyond $package, the $logCat of each call bubbles
-up thru the log-config, until it matches with your package-level
-config item.
+Log4perl's bubble-up behavior is still in effect; if your log-config
+never specifies anything beyond $package, the $logCat of each call
+bubbles up thru the log-config, until it matches with your
+package-level config item.
 
-In other words: since $logCat contains more information than that
-provided by :easy, your logging configuration can exersize more
-control over what gets logged.
+In other words: since the $logcat contains more information than that
+provided by :easy, your logging configuration can exersize more control
+over what gets logged.
 
 Note that you could have always used whatever categories you wanted,
 see L<Log::Log4perl/"Configuration files">, you just didnt get the
@@ -656,15 +654,16 @@ help that made it easy to do so.
 =head1 Test Coverage Results of your Application
 
 When your code uses this module, the logging calls are categorized and
-counted; this is reported (to standard out, if you don't override the
-default-config) when the program terminates.  The info is returned in
-3 chunks.
+counted; this is reported when the program terminates.  The info is
+returned in 3 chunks.
 
 =head2 Seen: How many times each invocation was called
 
 The %Seen hash stores and counts all invocations of all functions,
-keyed by the category.  This is variously called a usage report, a
-test-coverage report, a seen-report, etc.
+keyed by the category.  The keys are prefixed with '#log4perl.category.',
+which simplifies cut-paste of this output into your log-config.
+This is variously called a usage report, a test-coverage report, a
+seen-report, etc.
 
  (15:24:24.772) Logger.END.info.106: $Seen Log Events: = [
   {
@@ -681,9 +680,6 @@ Note that A.bar.debug.55 above has a negative count. This indicates
 that the logging activity was suppressed by the configuration, but the
 code was still reached 400 times.
 
-Also, the keys are prefixed with '#log4perl.category.', which
-simplifies cut-paste of this output into your log-config.
-
 =head2 UnSeen: What invocations were never called
 
 This report identifies the Logging invocations which were never called
@@ -720,10 +716,10 @@ you can tell what parts of the system are tested by each part of the
 test suite.
 
 Log4perl can write log-files with configurable names (ex: a
-timestamped name, see the L<SYNOPSIS> example above).  You can use
-this facility to write separate files for each sub-test, and then
-merge them (ex: `cat *.coverage-log|sort -u`), and quickly assess the
-total coverage provided by the entire test suite.
+timestamped name, see the L<SYNOPSIS> example).  You can use this
+facility to write separate files for each sub-test, and then merge
+them (ex: `cat *.coverage-log|sort -u`), and quickly assess the total
+coverage provided by the entire test suite.
 
 =head2 Keeping your Logging Config current
 
@@ -732,7 +728,7 @@ categories; this is more critical here, because categories are highly
 leveraged.  See L<Log::Log4perl/"Pitfalls with Categories"> for more
 info.
 
-This package is indisputably more vulnerable to such problems; it
+This package is undisputably more vulnerable to such problems; it
 exposes line numbers, which change regularly during development.  If
 you use numbers, be aware that adding comments to your code will
 change the category reported, rendering those log-config items
@@ -762,7 +758,7 @@ copying and editing the coverage report.  Just remove the quotes put
 in by Data::Dumper, change '=>' to '=', remove '#' to activate lines,
 and change the count to DEBUG, WARN, etc.
 
-=head2 What %Seen cannot tell you
+=head2 What %Seen cant tell you
 
 Because the %Seen coverage only includes the categories it sees at (a
 single) runtime, it cant 'remember' the aggregate coverage from
@@ -774,7 +770,8 @@ uncommented counterparts, but its a start.
 
 I may add an option to support a separate 'remembering' aggregate via
 a tied variable thats initialized in a BEGIN block.  This is a distant
-feature; I will spend some time with Devel::Cover to gain insight.
+feature; I need to find and copy prior art from somewhere in CPAN
+before I design and implement this.
 
 =head1 Filtering with Log::Log4perl::AutoCategorize
 
@@ -813,9 +810,8 @@ invocation.
 Note also that (4), even if it was uncommented, would have no effect
 on the logging, because its just saying that all debug() calls done by
 Frob::Vernier::twiddle() are issued at their default level.  Keeping
-such lines commented is good for documentation, and more efficient; it
-saves Log::Log4perl both cpu and memory by not loading config-items
-that arent meaningful.
+such lines commented is more efficient; it saves Log::Log4perl both
+cpu and memory by not loading config-items that arent meaningful.
 
 
 =head1 Other Features
@@ -922,15 +918,13 @@ The canonical way to get speed from AUTOLOAD is to not call it; or
 more helpfully, to create the desired subroutine so that it gets
 called for all future calls to that method.  But in this case, the
 decision reached in step 4 is different for every invocation, so it
-cannot be reduced to a single subroutine unless that routine calls
-caller(), which defeats the purpose.
+cannot be reduced to a single subroutine.
 
 The only way to get custom behavior for each invocation is to make
 them each invoke a unique method name; ie to munge the method names
 where they are invoked.  Once this is done, AUTOLOAD can then vivify
-subroutines which 'remember' results of caller(), and the log/nolog
-decision reachedd for that invocation.  Thereafter AUTOLOAD wont be
-called again (for that unique method), avoiding steps 0-4 completely.
+subroutines to implement them, and then it wont be called again (for
+that unique method), avoiding the 0-4 overhead completely.
 
 =head2 Using optimizer.pm
 
@@ -1066,42 +1060,21 @@ get wildly different results.
 =head1 CAVEAT (restated)
 
 This package expects you to use Logger->warn() style of coding,
-$logger->warn() will not work, unless 'my Dog $spot' syntax can make
-real guarantees.  This is because the type of $logger cannot be known
-at compile time, and its too risky to munge everything that looks like
-$flaregun->warn().
+$logger->warn() will not work, unless .  This is because the type of
+$logger cannot be known at compile time, and its too risky to munge
+everything that looks like $flaregun->warn().
 
 Even if it were possible, it would probably be bad to mix lexical code
 customization with objects and their attributes.  
 
 =head1 BUGS
 
-=head2 runaway process in Debugger 
-
-When I run a program using this module in debugger, and I hit
-control-P (Im an emacs user, so I do it habitually), the process runs
-amok, and I have to terminate it from another window.  If youre a vi
-user, this is less of a problem, but it warrants keeping this module
-in ALPHA state.  Its also the reason for the many debugging flags.
-
-=head2 abusable category extension
-
 The category extension can be abused; you can independently suppress
-warn() while elevating debug(), leading to surprises in what output
+warn() while elevating debug(), leading to suprises in what output
 goes where.  But this can be used sanely, so I dont consider worth
 disabling.  I could be convinced to optionally report it; as ever, a
 patch is a compelling argument.
 
-=head1 TODO
-
-=head2 quieter tests
-
-For ease of immediate use, this module defines a default logging
-config that writes to stdout.  This however makes the most basic tests
-also write to stdout, which is disconcerting for some would-be users.
-
-=head2 better test coverage support
-
 The %Seen report can never contain the categories of unexecuted
 functions, since the catalog entry is created by AUTOLOAD, which is
 never run if the function isnt executed.  Ive thought of a few
@@ -1116,16 +1089,19 @@ never run if the function isnt executed.  Ive thought of a few
        line-numbered log-specs, been moved by a mere comment. Note
        though that reset control could be added to API.
 
-Its probably occurred to you that Devel::Cover does all these things.
-It is however unsuitable for use in production code, and l4p's runtime
-reconfiguarability is quite valuable there.
+Debugging your code can be problematic; it seems to undo emacs
+keybindings; a ctrl-P (previous command) will send it into a
+cpu-sucking vortex.
 
+=head1 TODO
 
-=head2 better interoperation with l4p easy_init()
+=head2 fix raft of 'subroutine redefined' warnings
 
-Test t/09_coexist_easy.t demonstrates use of both l4p :easy, and l4pa,
-but it writes 2 separate logfiles.  Ive not tried recently to write to
-a single logfile.
+Warnings are issued (under -w) during compile-phase, I cant currently
+suppress them.  I suspect that solution lies in warnings::register, or
+perhaps in more primitive solutions like { package B::Generate; no
+warnings 'redefine' }.  If you can patch these away, Id be eternally
+grateful.
 
 =head2 devise good way to limit the deep logging of ref-data
 
@@ -1136,8 +1112,8 @@ the Dumper output, and/or print just the strings.
 But to do this via the log-config, the config-item syntax must be
 extended; either by adding new attributes, or by allowing new values.
 Either approach compromises interoperability with log4java config
-files, so these possible config-schemes should therefore be considered
-highly speculative (mostly here to elicit better ideas);
+files. These possible config-schemes should therefore be considered
+speculative;
 
     # using extra param
     log4perl.category.TestUtils.testcase_applies = DEBUG, 0
@@ -1219,10 +1195,10 @@ message issued.
 See also L</BUGS/Seen.2>.  While it borders on the baroque, its
 possible to tie %TotalInvocations to a DBM or Storable file, whose
 name is controlled by your log-config.  Then you could use Log4perl''s
-logitem = sub{} to synchronize the filename to your VERSION; either
+logitem = sub{} to syncronize the filename to your VERSION; either
 $package::VERSION, a complete CVS Revision, or to your application
 Version.  Interestingly, this could give some insight into longer-term
-code evolution.  See how Devel::Cover does it.
+code evolution.
 
 
 =head2 Improve the documentation
@@ -1234,15 +1210,10 @@ suggestions wrt what is tedious or unclear.
 
 To this end, Im including 2 versions of a lightning (5 min) talk I
 gave on this module at YAPC::NA in Boca.  The longer version I tested
-on several willing victims (thx guys), and the short version I
-actually gave based upon their valuable comments.  If these are
-helpful, please let me know how they could be folded into this
-document to improve clarity and/or remove redundancy.
-
-=head2 Provide patches to optimizer, B-Generate (DONE)
-
-To use this package on perl 5.8.1+ you need to patch both optimizer,
-B-Generate for updated internals.  I have patches on CPAN.
+on several willing victims, and the short version I actually gave
+based upon their valuable comments.  If these are helpful, please let
+me know how they could be folded into this document to improve clarity
+and/or remove redundancy.
 
 =head1 More TODO
 
@@ -1250,7 +1221,7 @@ B-Generate for updated internals.  I have patches on CPAN.
     write more test cases..
     optimize more
     integrate more fully with Log::Log4perl
-    revisit perftests
+
     help/feedback is welcome.
 
 =head1 CREDITS
@@ -1259,7 +1230,7 @@ Mike Schilli and Kevin Goess for L<Log::Log4perl>, and for reviewing
 my docs and code, and pointing out features I had not fully
 understood.
 
-Simon Cozens, Arthur Bergman for L<optimizer>, L<B::Generate>.  The
+Simon Cozens, Artur Bergman for L<optimizer>, L<B::Generate>.  The
 extra features this module provides would not be practical w/o the
 deep magic in those packages; Im just repeating incantations.
 
